/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.controller;

import ch.javablog.mobile.trakkcor.service.impl.HttpPostTrakkSender;
import java.util.Hashtable;

import javax.microedition.lcdui.Displayable;

import net.sf.microlog.Logger;
import net.sf.microlog.appender.FormAppender;
import ch.javablog.mobile.trakkcor.PlugInException;
import ch.javablog.mobile.trakkcor.TrakkcorMIDlet;
import ch.javablog.mobile.trakkcor.controller.action.Action;
import ch.javablog.mobile.trakkcor.model.BluetoothDeviceList;
import ch.javablog.mobile.trakkcor.service.GPSReceiver;
import ch.javablog.mobile.trakkcor.model.TrackList;
import ch.javablog.mobile.trakkcor.rms.Configuration;
import ch.javablog.mobile.trakkcor.rms.RMSTrackList;
import ch.javablog.mobile.trakkcor.service.impl.MockGPSReceiver;
import ch.javablog.mobile.trakkcor.service.impl.SimpleGPSReceiver;
import ch.javablog.mobile.trakkcor.ui.DeviceDiscoveryUI;
import ch.javablog.mobile.trakkcor.ui.GPSUI;
import ch.javablog.mobile.trakkcor.ui.InfoUI;
import ch.javablog.mobile.trakkcor.ui.LiveTrackerUI;
import ch.javablog.mobile.trakkcor.ui.LoadTrackUI;
import ch.javablog.mobile.trakkcor.ui.LogUI;
import ch.javablog.mobile.trakkcor.ui.OptionsUI;
import ch.javablog.mobile.trakkcor.ui.RemoveTrackUI;
import ch.javablog.mobile.trakkcor.ui.StartTrackUI;
import ch.javablog.mobile.trakkcor.ui.StartUI;
import ch.javablog.mobile.trakkcor.ui.TrackUI;

/**
 * The controller is the heart of this application. It holds every UI object and
 * is responsible for executing action. In addition the controller
 * implements the {@link ExecutionContext} interface which allows the action
 * to retrieve needed objects.
 * @author Heiko Maa
 */
public class Controller implements ExecutionContext {
    private final TrakkcorMIDlet midlet;
    private static Controller controller;
    private static final Logger LOG = Logger.getLogger();
    //-------------------------- Model
    private final BluetoothDeviceList     bluetoothDeviceList;
    private final RMSTrackList            trackList;
    private GPSReceiver             gpsReceiver;
    private final Configuration           configuration;
    
    //-------------------------- UI
    private final StartUI             startUI;
    private final StartTrackUI        startTrackUI;
    private final LoadTrackUI         loadTrackUI;
    private final RemoveTrackUI       removeTrackUI;
    private final TrackUI             trackUI;
    private final GPSUI               gpsUI;
    private final OptionsUI           optionsUI;
    private final DeviceDiscoveryUI   deviceDiscoveryUI;
    private final LogUI               logUI;
    private final InfoUI              infoUI;
    private final LiveTrackerUI       liveTrackerUI;
    
    //-------------------------- UI Mapping
    private final Hashtable uiMapping;
    
    /** Creates a new instance of Controller */
    private Controller(final TrakkcorMIDlet midlet) throws PlugInException {
        this.midlet = midlet;

        // Plugins
        configuration = new Configuration(new HttpPostTrakkSender());
        trackList = new RMSTrackList();
        configuration.init();
        trackList.init();
        
        bluetoothDeviceList = new BluetoothDeviceList();   
        
//#if ReleaseConfiguration
//#         gpsReceiver         = new SimpleGPSReceiver();
//#else
        gpsReceiver         = new MockGPSReceiver();
//#endif
        
        // UIs
        startUI             = new StartUI(this);
        gpsUI               = new GPSUI(this);
        optionsUI           = new OptionsUI(this);
        deviceDiscoveryUI   = new DeviceDiscoveryUI(this);
        logUI               = new LogUI(this);
        infoUI              = new InfoUI(this);
        startTrackUI        = new StartTrackUI(this);
        trackUI             = new TrackUI(this);
        loadTrackUI         = new LoadTrackUI(this);
        removeTrackUI       = new RemoveTrackUI(this);
        liveTrackerUI       = new LiveTrackerUI(this);
        bluetoothDeviceList.addListener(deviceDiscoveryUI);
        trackList.addListener(loadTrackUI);
        trackList.addListener(removeTrackUI);
       
        // mapping for UIs
        uiMapping = new Hashtable();
        uiMapping.put("start", startUI);
        uiMapping.put("startTrack", startTrackUI);
        uiMapping.put("loadTrack", loadTrackUI);
        uiMapping.put("removeTrack", removeTrackUI);
        uiMapping.put("track",trackUI);
        uiMapping.put("options", optionsUI);
        uiMapping.put("gps",  gpsUI);
        uiMapping.put("info",infoUI);
        uiMapping.put("deviceDiscovery",deviceDiscoveryUI);
        uiMapping.put("log",logUI);
        uiMapping.put("liveTracker", liveTrackerUI);

        // it is assumed that the form appender is configured in the property file
        FormAppender formAppender = (FormAppender) LOG.getAppender(0);
        formAppender.setLogForm(logUI);
    }
    
    /**
     * Gets the singleton instance of the Controller class
     */ 
    public static final synchronized Controller getInstance(final TrakkcorMIDlet midlet) throws PlugInException{
        if (controller == null) {
            controller = new Controller(midlet);
        } 
        return controller;
    }
    
    /**
     * Gets the GPSReceiver
     * @return
     */
    public GPSReceiver getGPSReceiver() {
        return gpsReceiver;
    }
    
    /**
     * Shows the Displayable object which is mapped to
     * the given name.  
     */ 
    public void forward(final String uiName) {
        // get UI
        Displayable d = (Displayable) uiMapping.get(uiName);
        midlet.showDisplayable(d);
    }
    
    public Displayable getDisplayable(final String uiName) {
        return (Displayable) uiMapping.get(uiName);
    }
    
    public Configuration getConfiguration() {
        return configuration;
    }
    
    public TrackList getTrackList() {
        return trackList;
    }
    
    public BluetoothDeviceList getBluetoothDeviceList() {
        return bluetoothDeviceList;
    }
    
    /**
     * Executes the given action in the same thread.
     * (Blocking operation)
     */ 
    public void executeAction(final Action action) {
        action.execute(this);
    }
    
    /**
     * Executes the given action within a new thread.
     * (Non-blocking operation)
     */ 
    public void executeBackgroundAction(final Action action) {
        LOG.debug(Controller.class.getName() + " executeBackground " + action.getClass().getName());
        BackgroundExecuter be = new BackgroundExecuter(action);
        Thread bgThread = new Thread(be);
        if (LOG.isDebugEnabled()) {
            LOG.debug("executeBackgroundAction: " + bgThread);
        }
        bgThread.start();
    }
    
    /**
     * Exits the app. PlugIns will be closed in order to 
     * guarantee a clean shutdown.
     */ 
    public void exit() {
        // close plugins
        try {
            configuration.close();
        } catch (PlugInException e) {
            LOG.error(e.toString());
        }
        
        try {
            trackList.close();
        } catch (PlugInException e) {
            LOG.error(e.toString());
        }
        this.midlet.destroyApp(true);
    }
    private final class BackgroundExecuter implements Runnable {
        private final Action action;
        public BackgroundExecuter(final Action action) {
            this.action = action;
        }
        
        public void run() {
            LOG.info("running in background: " + action.getClass().getName());
            action.execute(Controller.this);
        }
        
        public void cancel() {
            action.cancel(Controller.this);
        }
    }
    
}
