/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.controller.action;

import ch.javablog.mobile.trakkcor.controller.ExecutionContext;

/**
 *
 * @author Heiko Maa
 */
public interface Action {
    /**
     * Executes the action. 
     * @param context The context of this application so that other objects can be accessed.
     * @throws ActionException if something has gone wrong while executing the action.
     **/
    public void execute(ExecutionContext context) throws ActionException;
    /**
     * Submits a wish to cancel the current running action. Not all actions are willing to fulfil
     * this wish. This method is usually called from a another thread in order to 
     * cancel an operation which takes too much time.
     * The objects which were affected from executing must be restored to a consistent state.
     * @param context The context of this application so that other objects can be accessed.
     * @throws ActionException if something has gone wrong while executing the action.
     */
    public void cancel(ExecutionContext context) throws ActionException;
}
