 /*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.controller.action;

import ch.javablog.mobile.trakkcor.controller.ExecutionContext;
import ch.javablog.mobile.trakkcor.model.BluetoothDeviceList;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import net.sf.microlog.Logger;

/**
 * @author Heiko Maa
 */
public class DeviceDiscoveryPrepareAction implements Action {
    
    private final static Logger LOG = Logger.getLogger();
    public void execute(ExecutionContext context) {
        LOG.debug("execute DeviceDiscoveryPrepareAction...");
        try {
            final LocalDevice local = LocalDevice.getLocalDevice();
            final DiscoveryAgent agent = local.getDiscoveryAgent();
            final BluetoothDeviceList bluetoothDeviceList = context.getBluetoothDeviceList();
            final RemoteDevice[] cached   = agent.retrieveDevices(DiscoveryAgent.CACHED);
            final RemoteDevice[] preKnown = agent.retrieveDevices(DiscoveryAgent.PREKNOWN);
            
            bluetoothDeviceList.removeAllDevices();
            if (cached != null) {
                for (int i=0;i<cached.length;i++) {
                    bluetoothDeviceList.addDevice(cached[i]);
                }
            }
            if (preKnown != null) {
                for (int i=0;i<preKnown.length;i++) {
                    bluetoothDeviceList.addDevice(preKnown[i]);
                }
            }
        } catch (BluetoothStateException e) {
            e.printStackTrace();
        }
        LOG.debug("DeviceDiscoveryPrepareAction executed...");
       
    }
    
    public void cancel(ExecutionContext context) {
        
    }
    
}
