/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.controller.action;

import ch.javablog.mobile.trakkcor.controller.ExecutionContext;
import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.service.GPSReceiver;
import ch.javablog.mobile.trakkcor.rms.Configuration;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class GPSReceiveStartAction implements Action {
    private final GPSListener listener;
    private final static Logger LOG = Logger.getLogger();
    
    public GPSReceiveStartAction(GPSListener listener) {
        this.listener = listener;
    }

    public void execute(ExecutionContext context) {
        LOG.debug("GPSReceiveStartAction started...");
        
        // start GPS receiving
        final GPSReceiver receiver = context.getGPSReceiver();
        final Configuration configuration = context.getConfiguration();
        int bluetoothAccessPeriod = configuration.getBluetoothAccessPeriod();
        if (bluetoothAccessPeriod < 1) {
            bluetoothAccessPeriod = 1;
        }
        receiver.setBluetoothAddress(configuration.getBluetoothAddress());
        receiver.setNMEAMessageType(configuration.getNmeaMessageType());
        receiver.addListener(listener);
        receiver.start(bluetoothAccessPeriod * 1000);
        LOG.debug("GPSReceiveStartAction finished...");
    }

    public void cancel(ExecutionContext context) {
    }
    
}
