/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.model;

import ch.javablog.mobile.trakkcor.controller.listener.BluetoothDeviceListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.bluetooth.RemoteDevice;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class BluetoothDeviceList {
    
    private final Vector blutoothDevices;
    private final Vector listenerList;
    private volatile boolean isInquery;
    private final static Logger log = Logger.getLogger();
    /**
     * Creates a new instance of BluetoothDeviceList
     */
    public BluetoothDeviceList() {
        blutoothDevices = new Vector(5);
        listenerList = new Vector(1);
    }
    
    public void removeAllDevices() {
        blutoothDevices.removeAllElements();
        Enumeration listenerEnum = listenerList.elements();
        while (listenerEnum.hasMoreElements()) {
            BluetoothDeviceListener listener = (BluetoothDeviceListener) listenerEnum.nextElement();
            listener.allDevicesRemoved();
        }
    }
    
    public void addDevice(RemoteDevice device) {
        log.debug(this.getClass().getName() + " device added: " + device.getBluetoothAddress());
        blutoothDevices.addElement(device);
        Enumeration listenerEnum = listenerList.elements();
        while (listenerEnum.hasMoreElements()) {
            BluetoothDeviceListener listener = (BluetoothDeviceListener) listenerEnum.nextElement();
            listener.deviceAdded(device);
        }
    }
    public Enumeration devices() {
       return blutoothDevices.elements();
    }
    
    public void addListener(BluetoothDeviceListener listener) {
        log.debug(this.getClass().getName() + " listener added: " + listener);
        this.listenerList.addElement(listener);
    }
    
    public synchronized boolean isInquery() {
        return isInquery;
    }
    
    public synchronized void setInquery(boolean isInquery) {
       this.isInquery = isInquery; 
    }
}
