/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.model;

import ch.javablog.mobile.trakkcor.util.StringTokenizer;
import java.util.Date;

/**
 * RMC - Recommended Minimum Navigation Information
  Taken from http://www.nmea.de/nmea0183datensaetze.html#gprmc
 * 
 * <pre>
            1         2 3       4 5        6 7   8   9    10  11|
            |         | |       | |        | |   |   |    |   | |
     $--RMC,hhmmss.ss,A,llll.ll,a,yyyyy.yy,a,x.x,x.x,xxxx,x.x,a*hh<CR><LF>
     $GPRMC,191410.00,A,4735.5634,N,00739.3538,E,0.0,0.0,181102,0.4,E,A*19
         Field Number: 
          1) UTC Time
          2) Status, V = Navigation receiver warning
          3) Latitude
          4) N or S
          5) Longitude
          6) E or W
          7) Speed over ground, knots
          8) Track made good, degrees true
          9) Date, ddmmyy
         10) Magnetic Variation, degrees
         11) E or W
         12) Checksum 
 * </pre>
 * @author Heiko Maa
 */
public class GPRMCMessage implements NMEAMessage  {
    private String message;
    
    /** Universal Time Coordinated(UTC) **/
    private  Date utcDate;
    
    private  double latitude;
    
    /** longitude in decimal degrees **/  
    private  double longitude;
    
    /** speed over ground in knots */
    private double speedOverGround;
    
    /**
     *     A - ok,
     *     V - warning
     */
    private String status;
    
    
     public void parseMessage(final String message) {
        this.message = message;
        StringTokenizer tokenizer = new StringTokenizer(message,",");

        final String nmeaIdentifier              = tokenizer.nextToken();
        final String tmpUtcTime                  = tokenizer.nextToken();
        final String tmpStatus                   = tokenizer.nextToken();
        final String tmpNmeaLatitude             = tokenizer.nextToken();
        final String tmpDirectionNorthSouth      = tokenizer.nextToken();
        final String tmpNmeaLongitude            = tokenizer.nextToken();
        final String tmpDirectionEastWest        = tokenizer.nextToken();
        final String tmpSpeedOverGround          = tokenizer.nextToken();
        final String tmpTrackModeGood            = tokenizer.nextToken();
        final String tmpUtcDate                  = tokenizer.nextToken();
        final String tmpMagneticVariation        = tokenizer.nextToken();
        final String tmpChecksum                 = tokenizer.nextToken();
         
        /**
         * @todo utc date parsing.
         */
        this.utcDate = new Date();
        
        if (tmpStatus.equals("V")) {
            // don't parse because no valid signal
            this.message = "";
            return;
        }
        this.latitude = NMEAMessageParsingUtil.parseLatitude(tmpNmeaLatitude, tmpDirectionNorthSouth);
        this.longitude = NMEAMessageParsingUtil.parseLongitude(tmpNmeaLongitude, tmpDirectionEastWest);
        this.speedOverGround = NMEAMessageParsingUtil.parseTolerantDouble(tmpSpeedOverGround);
        this.status = tmpStatus;
    }

    public String getMessage() {
        return message;
    }

    public Date getUtcDate() {
        return utcDate;
    }

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    public double getSpeedOverGround() {
        return speedOverGround;
    }

    public String getStatus() {
        return status;
    }
    
    public boolean isValid() {
        return status != null && !status.equals("V");
    }

}
