/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package ch.javablog.mobile.trakkcor.model;

/**
 * "Where a numeric latitude or longitude is given, the two digits
 * immediately to the left of the decimal point are whole minutes, to the
 * right are decimals of minutes, and the remaining digits to the left of
 * the whole minutes are whole degrees.
 * 
 * Eg. 4533.35 is 45 degrees and 33.35 minutes. ".35" of a minute is
 * exactly 21 seconds." 
 * quote from http://aprs.gids.nl/nmea/
 * 
 * @author hmaass
 */
public class NMEAMessageParsingUtil {
    
    public static double parseTolerantDouble(final String toParse) {
        double parsedDouble = 0.0;
         try {
            parsedDouble= (Double.parseDouble(toParse));
        } catch (NumberFormatException e) {
            // ignore 
        }
        return parsedDouble;
    }
    
    public static int parseTolerantInt(final String toParse) {
        int parsedInt = 0;
         try {
            parsedInt= (Integer.parseInt(toParse));
        } catch (NumberFormatException e) {
            // ignore 
        }
        return parsedInt;
    }
    
    public static double parseLatitude(final String strLatitude, final String directionNorthSouth) {
        double latitude = 0.0;
        if (strLatitude.length() > 0) {
            int latitudePoint = strLatitude.indexOf(".");
            double latitudeDecimalMinutes = Double.parseDouble(strLatitude.substring(latitudePoint-2,strLatitude.length()));
            double latitudeDecimalDegree  = Double.parseDouble(strLatitude.substring(0,latitudePoint-2));
            latitude = latitudeDecimalDegree + latitudeDecimalMinutes / 60.0;
            if (directionNorthSouth.equals("S")) {
                latitude = - latitude;
            }
        }
        return latitude;
    }
    
    public static double parseLongitude(final String strLongitude, final String directionEastWest) {
        double longitude = 0.0;
        if (strLongitude.length() > 0) {
            int longitudePoint = strLongitude.indexOf(".");
            double longitudeDecimalMinutes = Double.parseDouble(strLongitude.substring(longitudePoint-2,strLongitude.length()));
            double longitudeDecimalDegree  = Double.parseDouble(strLongitude.substring(0,longitudePoint-2));
            longitude= longitudeDecimalDegree + longitudeDecimalMinutes / 60.0;

            if (directionEastWest.equals("W")) {
                longitude = - longitude;
            }
        } 
        return longitude;
    }    
}
