/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package ch.javablog.mobile.trakkcor.model;

import net.sf.microlog.Logger;

/**
 *
 * @author hmaass
 */
public class NMEAMessageType {
        private final String prefix;
        private final Class  nmeaMessageClazz;
        private final static Logger log = Logger.getLogger();
        public static final NMEAMessageType GPGGA = new NMEAMessageType("$GPGGA", GPGGAMessage.class);
        public static final NMEAMessageType GPRMC = new NMEAMessageType("$GPRMC", GPRMCMessage.class);

        private NMEAMessageType(final String prefix, final Class nmeaMessageClazz) {
            this.prefix = prefix;
            this.nmeaMessageClazz = nmeaMessageClazz;
        }
   
        public String getPrefix() {
            return prefix;
        }

        public Class getNmeaMessageClazz() {
            return nmeaMessageClazz;
        }
        
        public static NMEAMessageType findMessageType(final String message) {
            if (message.startsWith(GPGGA.prefix)) {
                return GPGGA;
            }
            if (message.startsWith(GPRMC.prefix) || message.startsWith("$--RMC")) {
                return GPRMC;
            }
            throw new IllegalArgumentException("could not find Message Type with message: " + message);
        }
        
        public static NMEAMessageType valueOf(final String prefix) {
            if (prefix.equals(GPGGA.prefix)) {
                return GPGGA;
            }
            if (prefix.equals(GPRMC.prefix)) {
                return GPRMC;
            }
            // fallback
            return GPGGA;
        }
        
        public NMEAMessage createNMEAMessage(final String message) {
            try {
                final NMEAMessage nmeaMessage = (NMEAMessage) this.nmeaMessageClazz.newInstance();
                nmeaMessage.parseMessage(message);
                return nmeaMessage;
            } catch (InstantiationException e) {
                log.error("error whil creating NMEAMessage: " +e);
            } catch (IllegalAccessException e) {
                log.error("error whil creating NMEAMessage: " +e);
            }
            // FIXME: what to return ?
            return null;
        }
    }
