/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.rms;

import ch.javablog.mobile.trakkcor.model.NMEAMessage;
import ch.javablog.mobile.trakkcor.model.NMEAMessageType;
import ch.javablog.mobile.trakkcor.model.Track;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import net.sf.microlog.Logger;

/**
 * A track contains a list of NMEAMessage objects, which
 * @author Heiko Maa
 */
public class RMSTrack implements Track {
    private final int id;
    private final Vector points;
    private final static long  serialVersion = 0x002;
    private final static Logger log = Logger.getLogger();
    /**
     * Creates a new instance of RMSTrack
     */
    public RMSTrack(final int id) {
        this.id = id;
        points = new Vector();
        
    }
    
    public RMSTrack(final byte[] serialized) throws RMSSerializeException  {
        
        ByteArrayInputStream bais = new ByteArrayInputStream(serialized);
        DataInputStream dis = new DataInputStream(bais);
        try {
            long serialVersion = dis.readLong();
            if (serialVersion != RMSTrack.serialVersion) {
                throw new IllegalArgumentException("serial version must be: " + RMSTrack.serialVersion);
            }
            points = new Vector();
            this.id = dis.readInt();
            int size = dis.readInt();
            for (int i = 0;i<size;i++){
                String message = dis.readUTF();
                NMEAMessageType messageType = NMEAMessageType.findMessageType(message);
                NMEAMessage nmeaMessage = messageType.createNMEAMessage(message);
                this.addPoint(nmeaMessage);
            }
        } catch (IOException io) {
            throw new RMSSerializeException(io);
        } finally {
            if (bais != null) {
                try {
                    bais.close();
                } catch (IOException io) {
                    throw new RMSSerializeException(io);
                }
            }
        }
    }
    
    /**
     * Only valid GGAInfo are stored, that means if qualityIndicator is 0
     * the ggaInfo will not be part of the track.
     */
    public void addPoint(final NMEAMessage ggaMessage) {
        if (ggaMessage.isValid()) {
            this.points.addElement(ggaMessage);
        }
    }
    
    /**
     * Gets an enumeration of GGAInfo objects
     */
    public Enumeration points() {
        return points.elements();
    }
    
    /**
     * Gets number of points stored in this track.
     **/
    public int size() {
        return points.size();
    }
    /**
     * Gets the id of the track
     */
    public int getId() {
        return this.id;
    }
    
    public static byte[] serialize(final Track t) throws RMSSerializeException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        final DataOutputStream dos = new DataOutputStream(baos);
        byte[] serializedTrack = null;
        try {
            log.debug(t.toString());
            dos.writeLong(serialVersion);
            dos.writeInt(t.getId());
            dos.writeInt(t.size());
            Enumeration points = t.points();
            while (points.hasMoreElements()) {
                NMEAMessage message = (NMEAMessage) points.nextElement();
                dos.writeUTF(message.getMessage());
            }
            serializedTrack = baos.toByteArray();
            
        } catch (IOException io) {
            throw new RMSSerializeException(io);
        } finally {
            try {
                baos.close();
            } catch (IOException io) {
                throw new RMSSerializeException(io);
            }
        }
        return serializedTrack;
    }
    public String toString() {
        return "id: " + id + "\n" +
               "points.size(): " + points.size();
    }
    

    
}
