/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.rms;

import ch.javablog.mobile.trakkcor.PlugInException;
import ch.javablog.mobile.trakkcor.controller.listener.TrackListListener;
import ch.javablog.mobile.trakkcor.model.Track;
import ch.javablog.mobile.trakkcor.model.TrackList;
import ch.javablog.mobile.trakkcor.model.TrackListException;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

/**
 *
 * @author Heiko Maa
 */
public class RMSTrackList implements TrackList {
    private RecordStore rsTrackList;
    private volatile boolean isOpen;
    private final Vector trackListListeners = new Vector();
    private final Vector ids = new Vector();
    
    public void init() throws PlugInException {
        try{
            this.rsTrackList = RecordStore.openRecordStore("RS_TRACKLIST",true);
            this.isOpen = true;
            
            // this will fill the ids vector
            this.allIds();
        } catch (Exception e) {
            throw new PlugInException(e);
        }
    }
    
  
    public void close() throws PlugInException {
        synchronized(this) {
            if (this.isOpen) {
                try {
                    this.rsTrackList.closeRecordStore();
                } catch (RecordStoreException re) {
                    throw new PlugInException(re);
                }
            }
        }
    }
    
    /**
     * Gets a track object from the RMS with the given id.
     */
    public Track getTrackById(final int id) throws TrackListException {
        synchronized (this) {
            try {
                final byte[] serializedTrack = this.rsTrackList.getRecord(id);
                RMSTrack track = new RMSTrack(serializedTrack);
                return track;
            } catch (RMSSerializeException e) {
                throw new TrackListException(e);
            } catch (RecordStoreException e) {
                throw new TrackListException(e);
            }
        }
    }
    
    /**
     * Creates a Track object with an entry in the RMS. You have to call #persistTrack
     * to save changes.
     *
     */
    public Track createTrack() throws TrackListException {
        synchronized (this) {
            try {
                final int trackID =this.rsTrackList.getNextRecordID();
                final RMSTrack track = new RMSTrack(trackID);
                final byte[] serializedTrack = RMSTrack.serialize(track);
                this.rsTrackList.addRecord(serializedTrack,0,serializedTrack.length);
                ids.addElement(new Integer(trackID));
                notifyListeners();
                return track;
            } catch (RecordStoreException e) {
                throw new TrackListException(e);
            } catch (RMSSerializeException e) {
                throw new TrackListException(e);
            }
        }
    }
    
    /**
     * Persists a track in the recordstore. The id of the given Track object
     * will be used as record id.
     */ 
    public void persistTrack(final Track track) throws TrackListException {
        final int trackID = track.getId();
        try {
            final byte[] serializedTrack = RMSTrack.serialize(track);
            this.rsTrackList.setRecord(trackID,serializedTrack,0,serializedTrack.length);
            
        } catch (RecordStoreException re) {
            throw new TrackListException(re);
        } catch (RMSSerializeException rs) {
            throw new TrackListException(rs);
        }
    }
    
    public Enumeration allIds() throws TrackListException {
        synchronized(this) {
            try {
                ids.removeAllElements();
                final RecordEnumeration recordEnum = this.rsTrackList.enumerateRecords(null,null,true);
                while (recordEnum.hasNextElement()) {
                    int id = recordEnum.nextRecordId();
                    ids.addElement(new Integer(id));
                }
            } catch (RecordStoreException e) {
                throw new TrackListException(e);
            }
        }
        return ids.elements();
    }
 
    public void removeTrack(final int id) throws TrackListException {
       synchronized(this) {
           try {
               this.rsTrackList.deleteRecord(id);
               ids.removeElement(new Integer(id));
           } catch (RecordStoreException e) {
               throw new TrackListException(e);
           }
           notifyListeners();
       }
    }
    
    public void removeAllTracks() {
        // @todo
    }
    
    public int size() throws TrackListException{
         synchronized(this) {
             try {
                return this.rsTrackList.getNumRecords();
             } catch (RecordStoreException re) {
                throw new TrackListException(re);
             }
            }
    }

    public void addListener(final TrackListListener listener) {
        trackListListeners.addElement(listener);
        // fill listener with ids
        listener.trackListModified(ids.elements());
    }
    
    private void notifyListeners() {
        final Enumeration enumListener = trackListListeners.elements();
        while(enumListener.hasMoreElements()) {
            TrackListListener listener = (TrackListListener)enumListener.nextElement();
            listener.trackListModified(ids.elements());
        }
    }
    
    
}
