/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.service.impl;

import ch.javablog.mobile.trakkcor.model.NMEAMessage;
import ch.javablog.mobile.trakkcor.model.Track;
import ch.javablog.mobile.trakkcor.service.TrakkSender;
import java.util.Enumeration;
import java.util.Vector;
import net.sf.microlog.Logger;

/**
 *
 * @author hmaass
 */
public abstract class AbstractTrakkSender implements TrakkSender {
private final static Logger LOG = Logger.getLogger();

    
    protected abstract void submit(String data, String URL );
    
    private final void send(final Enumeration points, final String URL, final String deviceID, final int trackID, final boolean isSinglePoint)  {
        LOG.debug("AbstractTrakkSender.send started...");
        
        if (points == null) {
            throw new IllegalArgumentException("points cannot be null");
        }
        if (deviceID == null) {
            throw new IllegalArgumentException("deviceID cannot be null");
        }
        
        if (URL == null) {
            throw new IllegalArgumentException("URL cannot be null");
        }
        
        // deviceID
        StringBuffer data = new StringBuffer("deviceID=");
        data.append(deviceID);

        // trackID
        data.append("&trackID="+trackID);

        // position
        String cgiParamPosition = null;
        if (isSinglePoint) {
            cgiParamPosition = "position";
        } else {
            cgiParamPosition = "position[]";
        }          
        while (points.hasMoreElements()) {
            NMEAMessage point = (NMEAMessage) points.nextElement();
            data.append("&"+cgiParamPosition+"="+point.getMessage());
        }
        LOG.debug("AbstractTrakkSender.send: data: " +data);
        this.submit(data.toString(), URL);
        LOG.debug("AbstractTrakkSender.send finished...");
    }
    
    public void sendTrakk(final Track track, final String URL, final String deviceID) {
        LOG.debug("AbstractTrakkSender.sendTrakk: track: " + track + " URL: " + URL + " deviceID: " + deviceID);
        this.send(track.points(), URL, deviceID,track.getId(),false);
    }
    
    public void sendSinglePoint(final NMEAMessage nmeaMessage, final String URL, final String deviceID) {
        LOG.debug("AbstractTrakkSender.sendTrakk: sendSinglePoint: " + nmeaMessage + " URL: " + URL + " deviceID: " + deviceID);
        final Vector v = new Vector();
        v.addElement(nmeaMessage);
        this.send(v.elements(),URL,deviceID, 0,true);
    }
}
