/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.service.impl;

import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.model.*;
import ch.javablog.mobile.trakkcor.service.GPSReceiver;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import net.sf.microlog.Logger;

/**
 * Simulates a GPSReceiver for testing reasons. The receiver always returns
 * for GPGGA: <pre>$GPGGA,130305.0,4717.115,N,00833.912,E,1,08,0.94,00499,M,047,M,,*\r\n"</pre> 
 * for GPRMC: <pre>$GPRMC,191410.00,A,4735.5634,N,00739.3538,E,0.0,0.0,181102,0.4,E,A*19</pre>
 */ 
public class MockGPSReceiver implements GPSReceiver {
    private final Vector listenerList;
    private final static Logger LOG = Logger.getLogger();
    private String bluetoothAddress;
    private volatile NMEAMessage nmeaMessage;
    private NMEAMessageType nmeaMessageType;
    private Timer timer;
    /** Creates a new instance of MockGPSReceiver */
    public MockGPSReceiver() {
        this.listenerList = new Vector(2);
   
    }
    public void setBluetoothAddress(String bluetoothAddress) {
        this.bluetoothAddress = bluetoothAddress;
    }

    public void addListener(GPSListener listener) {
        this.listenerList.addElement(listener);
    }
    
    public void removeListener(GPSListener listener) {
        this.listenerList.removeElement(listener);
    }
    
    public void notifyListeners() {
        Enumeration enumListener = listenerList.elements();
        while (enumListener.hasMoreElements()) {
            GPSListener listener = (GPSListener) enumListener.nextElement();
            if (this.nmeaMessageType == NMEAMessageType.GPGGA) {
                listener.GGAReceived((GPGGAMessage)nmeaMessage);
            }
            if (this.nmeaMessageType == NMEAMessageType.GPRMC) {
                listener.GPRMCReceived((GPRMCMessage)nmeaMessage);
            }
        }
    }

    public void start(long period) {
        if (bluetoothAddress == null) {
            throw new IllegalStateException("Bluetooth address is not set");
        }
        if (timer != null) {
            timer.cancel();
        }
        
        TimerTask gpsTask = new GPSTimerTask();
        timer = new Timer();
        timer.schedule(gpsTask,0,period);
    }
    
    public void stop() {
        if (timer != null) {
            timer.cancel();
        }
    }
    public void setNMEAMessageType(NMEAMessageType nmeaMessageType) {
         this.nmeaMessageType = nmeaMessageType;
    }
    
    private final class GPSTimerTask extends TimerTask {
         public void run() {
             LOG.debug("GPSTimerTask.run: " + nmeaMessage);
             if (nmeaMessageType == NMEAMessageType.GPGGA) {
                MockGPSReceiver.this.nmeaMessage = nmeaMessageType.createNMEAMessage("$GPGGA,130305.0,4717.115,N,00833.912,E,1,08,0.94,00499,M,047,M,,*");
             } else if (nmeaMessageType == NMEAMessageType.GPRMC){
                 MockGPSReceiver.this.nmeaMessage = nmeaMessageType.createNMEAMessage("$GPRMC,191410.00,A,4735.5634,N,00739.3538,E,0.0,0.0,181102,0.4,E,A*19");
             }
             notifyListeners();
         }
    }

}
