/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.ui;

import ch.javablog.mobile.trakkcor.controller.Controller;
import ch.javablog.mobile.trakkcor.controller.action.Action;
import ch.javablog.mobile.trakkcor.controller.action.DeviceDiscoverySearchAction;
import ch.javablog.mobile.trakkcor.controller.listener.BluetoothDeviceListener;
import java.io.IOException;
import java.util.Hashtable;
import javax.bluetooth.RemoteDevice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class DeviceDiscoveryUI extends List implements CommandListener, BluetoothDeviceListener {
    private final Command devicesCommand;
    private final Command selectCommand;
    private final Command backCommand;
    private final Controller controller;
    private final Hashtable nameToDevice;
    private final Logger LOG = Logger.getLogger();
    
    public DeviceDiscoveryUI(Controller controller) {
        super("Device List",List.IMPLICIT);
        this.controller = controller;
        this.nameToDevice = new Hashtable();
        devicesCommand = new Command("Query...", Command.SCREEN,2);
        selectCommand = new Command("Select", Command.ITEM,1);
        backCommand   = new Command("Back", Command.BACK,2);
        this.setSelectCommand(selectCommand);
        this.addCommand(devicesCommand);
        this.addCommand(backCommand);
        this.setCommandListener(this);
    }
   
    public final void commandAction(final Command c, final Displayable d) {
        if (c == selectCommand) {
            // retrieve selected remote device
            int selectedIndex = this.getSelectedIndex();
            String friendlyName = this.getString(selectedIndex);
            RemoteDevice remoteDevice =  (RemoteDevice) nameToDevice.get(friendlyName);
            controller.getConfiguration().setBluetoothAddress(remoteDevice.getBluetoothAddress());
            controller.forward("start");                       
        } else if (c == devicesCommand) {
            Action action = new DeviceDiscoverySearchAction();
            controller.executeAction(action);
        } else if (c == backCommand) {
           controller.forward("start");
        } 
        
    }
    
    // ----------------------------------------------- bluetooth device listener
    public void deviceAdded(RemoteDevice remoteDevice) {
    LOG.debug(this.getClass().getName() +  " device added: " + remoteDevice.getBluetoothAddress());
        String bluetoothAddress = remoteDevice.getBluetoothAddress();
        try {
            String friendlyName = remoteDevice.getFriendlyName(true);
            this.append(friendlyName,null);
            nameToDevice.put(friendlyName,remoteDevice);
        } catch (IOException io) {
            LOG.error(io.toString());
        }
    }
    
    
    public void allDevicesRemoved() {
        this.deleteAll();
        nameToDevice.clear();
    }
    
}
