/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.ui;

import ch.javablog.mobile.trakkcor.controller.Controller;
import ch.javablog.mobile.trakkcor.controller.action.Action;
import ch.javablog.mobile.trakkcor.controller.action.GPSReceiveStopAction;
import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.model.GPGGAMessage;
import ch.javablog.mobile.trakkcor.model.GPRMCMessage;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class GPSUI extends Form implements CommandListener, GPSListener {
    
    private Controller controller;
    private Command backCommand;
    private final static Logger log = Logger.getLogger();
    
    public GPSUI(final Controller controller) {
        super("GPS");
        this.controller = controller;
        backCommand = new Command("Back",Command.BACK,0);
        this.addCommand(backCommand);
        this.setCommandListener(this);
        this.append("waiting for GPS");
    }

    public void commandAction(final Command command, final Displayable d) {
        if (command == backCommand) {
            Action action = new GPSReceiveStopAction(this);
            controller.executeAction(action);
            controller.forward("start");
        } 
    }
    
    public void GGAReceived(final GPGGAMessage ggaMessage) {
        StringItem latitude = new StringItem("Latitude: ",String.valueOf(ggaMessage.getLatitude()));
        StringItem longitude = new StringItem("Longitude: ",String.valueOf(ggaMessage.getLongitude()));
        StringItem satellites = new StringItem("Satellites: ", String.valueOf(ggaMessage.getNumberOfSatellites()));
        StringItem altitude   = new StringItem("Altitude: ", String.valueOf(ggaMessage.getAltitude()));
        StringItem qualityIndicator = new StringItem("Quality: ", String.valueOf(ggaMessage.getQualityIndicator()));
        StringItem nmeaMessage      = new StringItem("NMEA: ", String.valueOf(ggaMessage.getMessage()));
        this.deleteAll();
        this.append(latitude);
        this.append(longitude);
        this.append(satellites);
        this.append(altitude);
        this.append(qualityIndicator);
        this.append(nmeaMessage);
    }

    public void GPRMCReceived(final GPRMCMessage gprmcMessage) {
        StringItem latitude = new StringItem("Latitude: ",String.valueOf(gprmcMessage.getLatitude()));
        StringItem longitude = new StringItem("Longitude: ",String.valueOf(gprmcMessage.getLongitude()));
        StringItem speedOverGround  = new StringItem("Speed (in knots): " , String.valueOf(gprmcMessage.getSpeedOverGround()));
        StringItem status           = new StringItem("Status: ", String.valueOf(gprmcMessage.getStatus()));
        StringItem nmeaMessage      = new StringItem("NMEA: ", String.valueOf(gprmcMessage.getMessage()));
        this.deleteAll();
        this.append(latitude);
        this.append(longitude);
        this.append(speedOverGround);
        this.append(status);
        this.append(nmeaMessage);
    }
    
}
