package ch.javablog.mobile.trakkcor.ui;

import java.util.Calendar;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.StringItem;

import ch.javablog.mobile.trakkcor.controller.Controller;
import ch.javablog.mobile.trakkcor.controller.action.GPSReceiveStartAction;
import ch.javablog.mobile.trakkcor.controller.action.GPSReceiveStopAction;
import ch.javablog.mobile.trakkcor.controller.action.SendLiveTrackAction;
import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.model.GPGGAMessage;
import ch.javablog.mobile.trakkcor.model.GPRMCMessage;

/**
 * @author Thomas Eichmann, namics ag
 **/
    
public class LiveTrackerUI extends Form implements CommandListener, GPSListener {
	private final Controller controller;
	
	private final Command backCommand;
	private final Command startCommand;
	private final Command stopCommand;
	private final StringItem status;
	private final StringItem time;
	private final StringItem altitude;
	private final StringItem longitude;
        private final StringItem latitude;
        private final StringItem speedOverGround;
	
	public LiveTrackerUI(Controller controller) {
		super("Live Tracker");

		this.controller   = controller;
		this.backCommand  = new Command("Back",  Command.BACK, 0);
		this.startCommand = new Command("Start", Command.ITEM, 1);
		this.stopCommand  = new Command("Stop",  Command.ITEM, 1);
		
		addCommand(this.backCommand);
		addCommand(this.startCommand);
		
		this.status    = new StringItem("Status ", "");
		this.time      = new StringItem("Time ", "");
		this.longitude = new StringItem("Longitude ", "");
		this.latitude  = new StringItem("Latitude ", "");
                this.altitude  = new StringItem("Altitude", "");
                this.speedOverGround = new StringItem("Speed","");
                
		
		// layout
		this.status.setLayout(Item.LAYOUT_NEWLINE_AFTER);
		this.time.setLayout(Item.LAYOUT_NEWLINE_AFTER);
		this.longitude.setLayout(Item.LAYOUT_NEWLINE_AFTER);
                this.latitude.setLayout(Item.LAYOUT_NEWLINE_AFTER);
		this.altitude.setLayout(Item.LAYOUT_NEWLINE_AFTER);
		
		append(this.status);
		append(this.time);
                append(this.latitude);
		append(this.longitude);
		append(this.altitude);
                append(this.speedOverGround);
		
		setCommandListener(this);
	}
	
	public void commandAction(Command command, Displayable displayable) {
		if(command == this.backCommand) {
			this.controller.forward("start");
		} else if(command == this.startCommand) {
			removeCommand(this.startCommand);
			addCommand(this.stopCommand);

			this.controller.executeAction(new GPSReceiveStartAction(this));
			
			setStatus("started");
			resetCoordinates();
		} else if(command == this.stopCommand) {
			removeCommand(this.stopCommand);
			addCommand(this.startCommand);
	
			this.controller.executeAction(new GPSReceiveStopAction(this));
			
			setStatus("stopped");
			resetCoordinates();
		}
	}

	public void GGAReceived(final GPGGAMessage ggaInfo) {
		this.longitude.setText(String.valueOf(ggaInfo.getLongitude()));
                this.latitude.setText(String.valueOf(ggaInfo.getLatitude()));
		this.speedOverGround.setText("");
                this.altitude.setText(String.valueOf(ggaInfo.getAltitude()));
		
		setStatus("sending data to server...");
		this.controller.executeAction(new SendLiveTrackAction(ggaInfo));
		setStatus("data sent");
	}
        
        public void GPRMCReceived(final GPRMCMessage gprmcInfo) {
		this.longitude.setText(String.valueOf(gprmcInfo.getLongitude()));
                this.latitude.setText(String.valueOf(gprmcInfo.getLatitude()));
		this.speedOverGround.setText(String.valueOf(gprmcInfo.getSpeedOverGround()));
                this.altitude.setText("");
		setStatus("sending data to server...");
		this.controller.executeAction(new SendLiveTrackAction(gprmcInfo));
		setStatus("data sent");
	}
	
	private void setStatus(String status) {
		this.status.setText(status);
		this.time.setText(currentTime());
	}
	
	private void resetCoordinates() {
                this.time.setText("");
		this.longitude.setText("");
                this.latitude.setText("");
		this.altitude.setText("");
                this.speedOverGround.setText("");
	}
	
	private String currentTime() {
		return Calendar.getInstance().getTime().toString();
	}

}
