/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.ui;

import ch.javablog.mobile.trakkcor.controller.Controller;
import ch.javablog.mobile.trakkcor.model.NMEAMessageType;
import ch.javablog.mobile.trakkcor.rms.Configuration;
import ch.javablog.mobile.trakkcor.service.TrakkSender;
import ch.javablog.mobile.trakkcor.service.impl.HttpGetTrakkSender;
import ch.javablog.mobile.trakkcor.service.impl.HttpPostTrakkSender;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.TextField;
import net.sf.microlog.Level;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class OptionsUI extends Form implements CommandListener{
    
    private final Controller controller;
    private final Command saveCommand;
    private final Command backCommand;
    private final TextField tfUrl;
    private final TextField tfBluetoothAccessPeriod;
    private final TextField tfId;
    private final ChoiceGroup cgTrakkSenderGroup;
    private final ChoiceGroup cgLogLevelGroup;
    private final ChoiceGroup cgNmeaMessageGroup;
    
    /** Creates a new instance of OptionsUI */
    private final Logger log = Logger.getLogger();
    
    public OptionsUI(final Controller controller) {
        super("Options");
        this.controller = controller;
        final Configuration configuration = controller.getConfiguration();
        final String url                = configuration.getUrl();
        final int bluetoothAccessPeriod = configuration.getBluetoothAccessPeriod();
        final String deviceId           = configuration.getDeviceId();
        final TrakkSender trakkSender   = configuration.getTrakkSender();
        final String logLevel           = configuration.getLogLevel();
        final NMEAMessageType nmeaMessageType    = configuration.getNmeaMessageType();
        
        saveCommand = new Command("Save", Command.OK,1);
        backCommand = new Command("Back", Command.BACK,2);
        
        this.addCommand(saveCommand);
        this.addCommand(backCommand);
        
        this.tfUrl                   = new TextField("URL", url,256,TextField.HYPERLINK);
        this.tfBluetoothAccessPeriod = new TextField("GPS refresh period (in s)", String.valueOf(bluetoothAccessPeriod), 4,TextField.DECIMAL );
        this.tfId                    = new TextField("DeviceID", deviceId, 20, TextField.ANY);
        
        // add values for HTTP Method
        this.cgTrakkSenderGroup       = new ChoiceGroup("HTTP Method",Choice.POPUP);
        this.cgTrakkSenderGroup.append("POST",null);
        this.cgTrakkSenderGroup.append("GET",null);
        
        // add values for Loglevel
        this.cgLogLevelGroup = new ChoiceGroup("Loglevel", Choice.POPUP);
        this.cgLogLevelGroup.append(Level.ERROR_STRING, null);
        this.cgLogLevelGroup.append(Level.WARN_STRING,null);
        this.cgLogLevelGroup.append(Level.INFO_STRING, null);
        this.cgLogLevelGroup.append(Level.DEBUG_STRING,null);

        // add values for NMEA MEssage
        this.cgNmeaMessageGroup = new ChoiceGroup("NMEA Message Type",Choice.POPUP);
        this.cgNmeaMessageGroup.append(NMEAMessageType.GPGGA.getPrefix(),null);
        this.cgNmeaMessageGroup.append(NMEAMessageType.GPRMC.getPrefix(),null);
        
        // select logLevel
        if (logLevel.equals(Level.ERROR_STRING)) {
            this.cgLogLevelGroup.setSelectedIndex(0,true);
        } else if (logLevel.equals(Level.WARN_STRING)) {
            this.cgLogLevelGroup.setSelectedIndex(1,true);
        } else if (logLevel.equals(Level.INFO_STRING)) {
            this.cgLogLevelGroup.setSelectedIndex(2,true);
        } else if (logLevel.equals(Level.DEBUG_STRING)) {
            this.cgLogLevelGroup.setSelectedIndex(3,true);
        }
        
        // select trakkSender
        if (trakkSender.toString().equals("POST")) {
            this.cgTrakkSenderGroup.setSelectedIndex(0,true);
        } else if (trakkSender.toString().equals("GET")) {
            this.cgTrakkSenderGroup.setSelectedIndex(1,true);
        }
        
        // select nmeaMessage
        if (nmeaMessageType.equals(NMEAMessageType.GPGGA)) {
            this.cgNmeaMessageGroup.setSelectedIndex(0,true);
        } else  if (nmeaMessageType.equals(NMEAMessageType.GPRMC)) {
            this.cgNmeaMessageGroup.setSelectedIndex(1,true);
        }

        
        this.append(this.tfUrl);
        this.append(this.tfBluetoothAccessPeriod);
        this.append(this.tfId);
        this.append(cgNmeaMessageGroup);
        this.append(cgTrakkSenderGroup);
        this.append(cgLogLevelGroup);
        this.setCommandListener(this);
    
    }
    public void commandAction(final Command command, final Displayable d) {
        if (command == saveCommand) {
            Configuration configuration = controller.getConfiguration();
            configuration.setUrl(tfUrl.getString());
            configuration.setDeviceId(tfId.getString());
            
            // TrakkSender 
            int trakkSenderIndex = this.cgTrakkSenderGroup.getSelectedIndex();
            if (trakkSenderIndex == 0) {
                configuration.setTrakkSender(new HttpPostTrakkSender());
            } else {
                configuration.setTrakkSender(new HttpGetTrakkSender());
            }
            
            // Loglevel
            int logLevelIndex = this.cgLogLevelGroup.getSelectedIndex();
            switch (logLevelIndex) {
                case 0: configuration.setLogLevel(Level.ERROR_STRING); break;
                case 1: configuration.setLogLevel(Level.WARN_STRING); break;
                case 2: configuration.setLogLevel(Level.INFO_STRING); break;
                case 3: configuration.setLogLevel(Level.DEBUG_STRING); break;
            }

            // nmeaMessage
            int nmeaMessageIndex = this.cgNmeaMessageGroup.getSelectedIndex();
            switch (nmeaMessageIndex) {
                case 0: configuration.setNmeaMessageType(NMEAMessageType.GPGGA); break;
                case 1: configuration.setNmeaMessageType(NMEAMessageType.GPRMC); break;
            }
            
            int bluetoothAccessPeriod = 60;
            try {
                bluetoothAccessPeriod = Integer.parseInt(this.tfBluetoothAccessPeriod.getString());
            } catch (NumberFormatException n) {
                log.error("bluetoothAccessPeriod is not a valid number");
            }

            configuration.setBluetoothAccessPeriod(bluetoothAccessPeriod);
            log.info("configuration url:= " + tfUrl.getString());
            log.info("configuration bluetoothAccessPeriod:= " + bluetoothAccessPeriod);
        } 
         controller.forward("start");

    }
}
