/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package ch.javablog.mobile.trakkcor.ui;

import ch.javablog.mobile.trakkcor.controller.Controller;
import ch.javablog.mobile.trakkcor.controller.action.GPSReceiveStartAction;
import ch.javablog.mobile.trakkcor.controller.action.GPSReceiveStopAction;
import ch.javablog.mobile.trakkcor.controller.action.PersistTrackAction;
import ch.javablog.mobile.trakkcor.controller.action.SendTrackAction;
import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.model.GPGGAMessage;
import ch.javablog.mobile.trakkcor.model.GPRMCMessage;
import ch.javablog.mobile.trakkcor.model.Track;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;

/**
 *
 * @author Heiko Maa
 */
public class TrackUI extends Form implements CommandListener, GPSListener {
    private final Controller controller;
    private Track track;
    private Command saveCommand;
    private Command backCommand;
    private Command sendCommand;
    private Command startCommand;
    private Command stopCommand;
    
    public TrackUI(Controller controller) {
        super("Track");
        this.controller = controller;
        backCommand = new Command("Back",Command.BACK,1);
        startCommand =  new Command("Start Receiving",Command.OK,1);
        stopCommand  = new Command("Stop Receiving",Command.OK,1);
        saveCommand   = new Command("Save Track",Command.OK,0);
        sendCommand   = new Command("Send Track", Command.OK,0);
        
        this.addCommand(saveCommand);
        this.addCommand(backCommand);
        this.addCommand(sendCommand);
        this.addCommand(startCommand);
        this.setCommandListener(this);
    }
    
    public void commandAction(Command command, Displayable displayable) {
        if (command == backCommand) {
            // stop listening and don't save track
            GPSReceiveStopAction rsa = new GPSReceiveStopAction(this);
            controller.executeAction(rsa);    
            allowStartCommand();
            controller.forward("startTrack");
        } else if (command == saveCommand) {
            GPSReceiveStopAction rsa = new GPSReceiveStopAction(this);
            controller.executeAction(rsa);
            PersistTrackAction pta = new PersistTrackAction(this.track);
            controller.executeAction(pta);
            allowStartCommand();
        } else if (command == sendCommand) {
            // W800i bug:  gps receiving must be stopped before sending data to the internet
            GPSReceiveStopAction rsa = new GPSReceiveStopAction(this);
            controller.executeAction(rsa);
            SendTrackAction sta = new SendTrackAction(this.track);
            controller.executeBackgroundAction(sta);
            allowStartCommand();
        } else if (command == startCommand) {
            GPSReceiveStartAction rsa = new GPSReceiveStartAction(this);
            controller.executeAction(rsa);
            allowStopCommand();
        } else if (command == stopCommand) {
            GPSReceiveStopAction rsa = new GPSReceiveStopAction(this);
            controller.executeAction(rsa);
            allowStopCommand();
        }
    } 
    private void allowStartCommand() {
        this.removeCommand(stopCommand);
        this.addCommand(startCommand);
       
        
    }
    private void allowStopCommand() {
        this.removeCommand(startCommand);
        this.addCommand(stopCommand);
    } 
       
    public void GGAReceived(GPGGAMessage ggaInfo) {
        this.track.addPoint(ggaInfo);
        this.deleteAll();
        this.append(new StringItem("id: ", "" + track.getId()));
        this.append(new StringItem("points: ", "" + track.size()));
        this.append(new StringItem("current: ", ggaInfo.getMessage()));
    }
    public void GPRMCReceived(GPRMCMessage gprmcInfo) {
        this.track.addPoint(gprmcInfo);
        this.deleteAll();
        this.append(new StringItem("id: ", "" + track.getId()));
        this.append(new StringItem("points: ", "" + track.size()));
        this.append(new StringItem("current: ", gprmcInfo.getMessage()));
    }
    
    public void setTrack(Track track) {
        this.track = track;
        this.deleteAll();
        this.append(new StringItem("id: ", "" + track.getId()));
        this.append(new StringItem("points: ", "" + track.size()));
    }
    
    
}
